#[derive(Debug)]
pub enum MoveOutcome<P> {
    Win(P),
    Tie,
    Pass,
}

pub trait GamePlayer: Copy + Clone + Eq + Send + Sync {
    fn first() -> Self;
    fn next(&self) -> Self;
}

pub trait GameMove: Copy + Clone + Send + Sync {}

pub trait Game<P: GamePlayer, M: GameMove>: Clone + Send + Sync {
    type Moves<'a>: Iterator<Item = M> + Sized
    where
        Self: 'a;
    fn current_player(&self) -> P;
    fn moves<'a>(&'a self) -> Self::Moves<'a>;
    fn apply(&mut self, m: M) -> Option<MoveOutcome<P>>;
}
